#!/bin/sh

# cdburn.sh <isofile> [--md5 file] [--dev device]

# Josh's CD Burning Script!
# Burns the given ISO file to CD and verifies it was burned correctly.
# Can also verify the CD matches the given MD5 file.
# If autodetection fails, you can also give a cdrecord device string.

###############
#
#	Editable stuff here
#
###############

# Directory on large partition for temp file (need 0.7GB free)
TEMPPREFIX="/var/tmp"

# These defaults should be fine, no real need to edit
CDRECORD="/usr/bin/cdrecord"
CDROPTS="-v -dao -pad -eject"
READCD="/usr/bin/readcd"
READOPTS="-v"
DIFF="/usr/bin/diff"
MEAN="/bin/nice --99"

###############
#
#	Here be dragons
#
###############

showUsage() {
	echo "Usage: $0 <isofile>"
	echo "       --help         = show this help"
	echo "       --md5 <file>   = optional MD5 file for CD verification"
	echo "       --dev <device> = cdrecord dev to use (override autodetection)"
	echo "       --compare      = do NOT burn, instead compare already-burned CD to ISO"
}

ISOFILE=$1
shift
if [ "x${ISOFILE}" = "x" -o "x${ISOFILE}" = "x--help" ]; then
	showUsage
	exit 1
fi

MD5FILE=""
DEVICE=""
COMPARE=""
while [ "x$1" != "x" ]
do
	ARG=$1
	shift
	
	if [ "x$ARG" = "x--md5" ]; then
		MD5FILE=$1
		shift
	elif [ "x$ARG" = "x--dev" ]; then
		DEVICE=$1
		shift
	elif [ "x$ARG" = "x--compare" ]; then
		COMPARE="yes"
	else
		showUsage
		exit 1
	fi
done

# Sanity check ISO file
if [ "x${ISOFILE}" = "x" ]; then
	showUsage
	exit 1
fi
if [ ! -r ${ISOFILE} ]; then
	echo "Unable to read ISO file ${ISOFILE}"
	exit 1
fi
ISOSIZE=`/bin/ls -l ${ISOFILE} | awk '{ print $5 }'`
if [ "${ISOSIZE}" -le 0 ]; then
	echo "Problem determining size of ISO file ${ISOFILE}"
	exit 1
fi
ISOSECTORS=`expr ${ISOSIZE} / 2048`
BYTECOUNT=`expr ${ISOSECTORS} \* 2048`
if [ "${ISOSIZE}" != "${BYTECOUNT}" ]; then
	echo "ISO file size not valid - corrupted file?"
	exit 1
fi
echo "ISO file ${ISOFILE}"

# If user gave a MD5 file, sanity check it also
if [ "x${MD5FILE}" != "x" ]; then
	if [ ! -r ${MD5FILE} ]; then
		echo "Unable to read MD5 file ${MD5FILE}"
		exit 1
	fi
	MD5TEMP=`cat ${MD5FILE} | grep ${ISOFILE}`
	if [ "$?" != "0" ]; then
		echo "ISO file not mentioned in MD5 file!"
		exit 1
	fi
	MD5GOAL=`echo ${MD5TEMP} | head -n 1 | awk '{ print $1 }'`
	if [ "x${MD5GOAL}" != "x" ]; then
		echo "MD5 file ${MD5FILE}"
	else
		echo "Unable to parse MD5 file!"
		exit 1
	fi
fi

# Autodetect CD burner device if user didn't specify it manually
if [ "x${DEVICE}" = "x" ]; then
	echo "Autodetecting CD burner"
	
	TEMPFILE1=${TEMPPREFIX}/cdburn-1$$
	TEMPFILE2=${TEMPPREFIX}/cdburn-2$$
	
	touch $TEMPFILE1
	touch $TEMPFILE2
	if [ ! -w "${TEMPFILE1}" ]; then
		echo "Unable to create temporary file ${TEMPFILE1}!"
		exit 1
	fi
	if [ ! -w "${TEMPFILE2}" ]; then
		echo "Unable to create temporary file ${TEMPFILE2}!"
		rm ${TEMPFILE1}
		exit 1
	fi
	
	${CDRECORD} -scanbus >${TEMPFILE1} 2>&1
	if [ "$?" != "0" ]; then
		echo "SCSI device scan failed!"
		echo "Are you root?"
		echo "Did you set kernel command line hdc=ide-scsi (or something like that)?"
		rm ${TEMPFILE1} ${TEMPFILE2}
		exit 1
	fi
	
	# Crude heuristic: lines containing 3 numbers like "0,0,0", and "CD"
	CDLINES=`cat ${TEMPFILE1} | grep CD | grep '[0-9]*,[0-9]*,[0-9]*' | awk '{ print $1 }'`
	
	# A real SCSI system might have devices that aren't burners, so skip them
	for DEVTEST in $CDLINES
	do
		${CDRECORD} -dev=${DEVTEST} -checkdrive >${TEMPFILE2} 2>&1
		if [ "$?" = "0" ]; then
			CHECKDRIVE=`cat ${TEMPFILE2}`
			echo ${CHECKDRIVE} | grep 'Supported modes' | grep 'SAO' >/dev/null
			if [ "$?" = "0" ]; then
				DEVICE=${DEVTEST}
				break
			fi
		fi
	done
	
	# As a courtesy, regenerate scan output for user if autodetect fails
	if [ "x${DEVICE}" = "x" ]; then
		echo "Could not detect your CD burner!"
		echo "Try again and specify it manually?"
		echo
		cat ${TEMPFILE1}
		rm ${TEMPFILE1} ${TEMPFILE2}
		exit 1
	fi

	echo "Found device ${DEVICE}"
	rm ${TEMPFILE1} ${TEMPFILE2}
fi

# If user wants MD5, do it now (to avoid burning a coaster)
if [ "x$MD5GOAL" != "x" ]; then
	echo "Testing MD5 of ${ISOFILE}"
	MD5NEW=`md5sum ${ISOFILE} | awk '{ print $1 }'`
	if [ "x$MD5NEW" != "x$MD5GOAL" ]; then
		echo "MD5 CHECKSUMS DO NOT MATCH!"
		exit 1
	else
		echo "MD5 is good"
	fi
fi

# See if we can use Burn-Proof
EXTRAOPTS=""
BURNFREE=`echo ${CHECKDRIVE} | grep "Driver flags" | grep "BURNFREE"`
if [ "$?" = "0" ]; then
	EXTRAOPTS="driveropts=burnfree"
fi

if [ "x$COMPARE" != "x" ]; then
	echo "Not burning CD - About to verify and compare already-burned CD to ISO"
else
	# Test for existence of a burnable disk
	CHECKDISK=`${CDRECORD} -dev=${DEVICE} -atip 2>&1`
	if [ "$?" != "0" ]; then
		echo "Please put a blank disk in the drive!"
		exit 1
	fi

	# Test to see if nice is mean enough
	CHECKNICE=`${MEAN} true`
	if [ "$?" != "0" ]; then
		echo "Unable to raise priority.  Are you root?"
		exit 1
	fi

	# Here we go!
	echo
	echo "====================="
	echo "STARTING BURN OF DISC"
	echo "====================="
	echo
	${MEAN} ${CDRECORD} -dev=${DEVICE} ${CDROPTS} ${EXTRAOPTS} ${ISOFILE}
	if [ "$?" != "0" ]; then
		echo "DOH!"
		exit 1
	fi
fi

# Test validity of device and close drive door if it is open
echo
echo "==================================="
echo "DO NOT TOUCH DISC - STARTING VERIFY"
echo "==================================="
echo
CHECKLOAD=`${CDRECORD} -dev=${DEVICE} -load 2>/dev/null`
if [ "$?" != "0" ]; then
	echo "Problem reloading device ${DEVICE}!"
	exit 1
fi

# Build temp file location
READBACKFILE=${TEMPPREFIX}/cdburn-$$.iso

# Sanity check temp file location
touch ${READBACKFILE}
if [ ! -w ${READBACKFILE} ]; then
	echo "Problem creating temporary file ${READBACKFILE}"
	rm -f ${READBACKFILE}
	exit 1
fi

# Here we go!
echo "Reading CD to ${READBACKFILE}"
echo
${READCD} -dev=${DEVICE} ${READOPTS} f=${READBACKFILE} sectors=0-${ISOSECTORS}
if [ "$?" != "0" ]; then
	echo "There was an error reading from the CD!"
	rm -f ${READBACKFILE}
	exit 1
fi
echo

# Do the comparison
echo "Comparing CD to ${ISOFILE}"
${DIFF} -q ${READBACKFILE} ${ISOFILE} >/dev/null 2>&1
if [ "$?" != "0" ]; then
	echo "THE CD DOES NOT MATCH!"
	rm -f ${READBACKFILE}
	exit 1
fi

rm -f ${READBACKFILE}
echo "SWEET!"

# Eject the CD as a convenience only (ignore errors)
${CDRECORD} -dev=${DEVICE} -eject >/dev/null 2>&1
echo
echo "============================"
echo "ALL DONE - OK TO REMOVE DISC"
echo "============================"
echo
exit 0
